package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.DateUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.dao.EvaluationPortalMapper;
import com.ims.eval.entity.EvaluationPortal;
import com.ims.eval.entity.EvaluationPortalAttach;
import com.ims.eval.entity.OrganizationEvaluation;
import com.ims.eval.service.IEvaluationPortalAttachService;
import com.ims.eval.service.IEvaluationPortalService;
import com.ims.eval.service.IEvaluationScoreCountService;
import com.ims.eval.service.IOrganizationEvaluationService;
import groovy.util.logging.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 考评首页 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-05-10
 */
@Slf4j
@Service
public class EvaluationPortalServiceImpl extends ServiceImpl<EvaluationPortalMapper, EvaluationPortal> implements IEvaluationPortalService {


	@Autowired
	private IEvaluationPortalAttachService evaluationPortalAttachService;

	@Autowired
	private IOrganizationEvaluationService organizationEvaluationService;

	@Autowired
	private IEvaluationScoreCountService evaluationScoreCountService;

//	@Autowired
//	private EvaluationPortalMapper evaluationPortalMapper;

	@Override
	public Map<String, Object> getPortalData(String binSection, String year, String season, String newyear) {
		Map<String, Object> map = new HashMap<>();
		Calendar date = Calendar.getInstance();
		try {
			List<Map<String, Object>> yearList = baseMapper.selectPortalList(binSection, date.get(Calendar.YEAR) == Integer.valueOf(newyear) ? String.valueOf(Integer.valueOf(newyear) - 1) : newyear, null);
			List<Map<String, Object>> seasonList = baseMapper.selectPortalList(binSection, year, season);
			Map<Object, List<Map<String, Object>>> yearMap = yearList.stream().collect(Collectors.groupingBy(e -> e.get("bin_section"), Collectors.toList()));
			Map<Object, List<Map<String, Object>>> seasonMap = seasonList.stream().collect(Collectors.groupingBy(e -> e.get("bin_section"), Collectors.toList()));
			map.put("year", yearMap);
			map.put("season", seasonMap);
		} catch (Exception e) {
			log.error(e.getMessage());
		}

		return map;
	}

	@Override
	public List<EvaluationPortal> getEvaluationPortalList(String year, String season, String binSection, String memo1) {

		List<EvaluationPortal> list = baseMapper.selectEvaluationPortalList(year, season, binSection, memo1);


		return list;
	}

	@Transactional
	@Override
	public boolean saveSeasonPerformanceRanking(EvaluationPortal portal, EvaluationPortalAttach attach) {
		if ("FD".equals(portal.getBinSection())) {
			portal.setBinSection("XNYFG");
		}
		List<EvaluationPortal> portals = getEvaluationPortalList(portal.getYear(), portal.getSeason(), portal.getBinSection(), portal.getMemo1());
		if (null != portals && portals.size() > 0) {
			List<String> idList = portals.stream().map(EvaluationPortal::getId).collect(Collectors.toList());
			boolean b = this.removeByIds(idList);
			if (!b) {
				throw new CustomException("操作失败");
			}
			if (null != idList && idList.size() > 0) {
				b = evaluationPortalAttachService.removeEvaluationPortalAttach("", idList);
				if (!b) {
					throw new CustomException("操作失败");
				}
			}
		}

		boolean b1 = this.saveOrUpdate(portal);
		if (!b1) {
			throw new CustomException("操作失败");
		}
		attach.setPortalId(portal.getId());
		boolean b2 = evaluationPortalAttachService.saveOrUpdate(attach);
		if (!b2) {
			throw new CustomException("操作失败");
		}
		return b2;
	}

	@Override
	public void rating(String OrganizationEvaluation) {
		OrganizationEvaluation evaluation = null;
		if (StringUtils.isEmpty(OrganizationEvaluation)) {
			//获取当前日期
			String year = String.valueOf(DateUtils.getYear(new Date()));
			String season = String.valueOf(DateUtils.getSeason(new Date()));
			evaluation = organizationEvaluationService.list(OrganizationEvaluation, "JDKP", year, season).get(0);
		} else {
			evaluation = organizationEvaluationService.getById(OrganizationEvaluation);
		}
		//按照板块获取分数

		evaluationScoreCountService.getListByEvaluationRuleId("", "", "");
		evaluation.getId();

	}

	@Override
	public IPage<EvaluationPortal> listPage(String binSection, String year, String season) {
		return null;
	}

	@Override
	public Map<String, Object> selectPortalList(String binSection, String year, String season) {
		Map<String, Object> map = new HashMap<>();
		Calendar date = Calendar.getInstance();
		try {
			List<Map<String, Object>> yearList = baseMapper.selectPortalList(binSection, date.get(Calendar.YEAR) == Integer.valueOf(year) ? String.valueOf(Integer.valueOf(year) - 1) : year, null);
			Map<Object, List<Map<String, Object>>> yearMap = yearList.stream().collect(Collectors.groupingBy(e -> e.get("bin_section"), Collectors.toList()));
			map.put("year", yearMap);
		} catch (Exception e) {
			log.error(e.getMessage());
		}
		return map;
	}

	@Override
	public Map<String, String> getHomeSeason() {
		Map<String, String> hs = new HashMap<>();
		EvaluationPortal homeSeason = null;
		EvaluationPortal homeYear = null;
		try {
			LambdaQueryWrapper<OrganizationEvaluation> lqw = new LambdaQueryWrapper<>();
			LambdaQueryWrapper<OrganizationEvaluation> eq = lqw.eq(OrganizationEvaluation::getStage, "流程已结束");
			List<OrganizationEvaluation> list = organizationEvaluationService.list(eq);
			if (list == null || list.isEmpty()) {
				hs.put("newyear", "2022");
				hs.put("year", "");
				hs.put("season", "");
				return hs;
			}
			Map<String, List<OrganizationEvaluation>> cc = list.stream().collect(Collectors.groupingBy(OrganizationEvaluation::getCheckCycle));
			List<OrganizationEvaluation> jdkp = cc.get("JDKP");
			List<OrganizationEvaluation> ndkp = cc.get("NDKP");
			if ((jdkp == null || jdkp.isEmpty()) && (ndkp == null || ndkp.isEmpty())) {
				hs.put("newyear", "2022");
				hs.put("year", "");
				hs.put("season", "");
				return hs;
			}
			if (jdkp == null || jdkp.isEmpty()) {
				hs.put("year", "");
				hs.put("season", "");
			} else if (jdkp.size() == 1) {
				hs.put("year", jdkp.get(0).getYear());
				hs.put("season", jdkp.get(0).getMonth());
			} else {
				Comparator<OrganizationEvaluation> comparator = Comparator.comparing(OrganizationEvaluation::getYear).thenComparing(OrganizationEvaluation::getMonth);
				Collections.sort(jdkp, comparator.reversed());
				OrganizationEvaluation latestData = jdkp.get(0);
				hs.put("year", latestData.getYear());
				hs.put("season", latestData.getMonth());
			}
			if (ndkp == null || ndkp.isEmpty()) {
				hs.put("newyear", "2022");
			} else if (ndkp.size() == 1) {
				hs.put("newyear", ndkp.get(0).getYear());
			} else {
				Comparator<OrganizationEvaluation> comparator = Comparator.comparing(OrganizationEvaluation::getYear);
				Collections.sort(ndkp, comparator.reversed());
				OrganizationEvaluation latestData = ndkp.get(0);
				hs.put("newyear", latestData.getYear());
			}

//			homeSeason = evaluationPortalMapper.getHomeSeason();
//			homeYear = evaluationPortalMapper.getHomeYear();
		} catch (Exception e) {
			e.printStackTrace();
		}
//
//		if(null != homeYear){
//			hs.put("newyear",homeYear.getYear());
//		}
//
//		if(null != homeSeason){
//			hs.put("year",homeSeason.getYear());
//			hs.put("season",homeSeason.getSeason());
//		}else {
//			hs.put("year",DateUtils.getYear(new Date())+"");
//			hs.put("season",DateUtils.getSeason(new Date())+"");
//		}

		return hs;
	}
}
